#!/usr/bin/env python

from __future__ import nested_scopes
import string, sys, os

from Ciphon import install, config, check, ftplib2

if sys.version_info[0] == 2 and sys.version_info[1] >= 2:
    import ftplib
else:
    from Ciphon import ftplib2 as ftplib
    
__version__ = "$Revision: 1.10 $"

class ReadEvalLoop:
    """Class to run main read-eval loop"""
    
    def __init__(self):
        """Initialize class variables"""

        self.__commands = {'help': self.__mainHelp,
                           'quit': self.__quit,
                           'exit': self.__quit}
        self.__help = {'quit': 'Exit ciphon',
                       'exit': 'Exit ciphon',
                       'help': 'help'}
        self.__matches = []
        
    def __getCompletion(self, text, state):
        """Function to return the next completion for text"""

        if state == 0:
            self.__matches = []
            length = len(text)
            # slightly hairy list comprehension that returns the commands that
            # match text
            for word in [x for x in self.__commands.keys() if x[:length] == text]:
                self.__matches.append(word)
        
        try:
            return self.__matches[state]
        except IndexError:
            return None
        

    def begin(self):
        """start read-eval loop that reads input and dispatches based on that"""
        sys.stdout.write( "Ciphon 0.3.5 Copyright 2001 Suchandra Thapa\n")
        sys.stdout.write("See license.txt for license information\n")

        if config.getConfig().getParameter('readline') == 'true':
            import readline
            readline.parse_and_bind("tab: complete")
            readline.set_completer(self.__getCompletion)
        
        while 1:
            # basically the read-eval loop works by reading the first word
            # of the user's input, if this word is a key in  self.__commands, the
            # function associated with that key is run on the rest of the user's
            # input. Otherwise the help function is run
            input = ''
            try:
                input = raw_input('ciphon> ')
            except EOFError:
                sys.exit(0)
            input = string.split(input)
            if len(input) == 0:
                input = ['help']
            if input[0] in self.__commands.keys():
                dispatch = self.__commands[input[0]]
            else:
                dispatch = self.__mainHelp

            dispatch(input)

    def __mainHelp(self, input):
        """Displays help for main eval loop and commands"""
        if len(input) == 1 or not (input[1] in self.__help.keys()):
            sys.stderr.write("\nAllowed commands: \n")
            for key in self.__help.keys():
                sys.stderr.write("%s\n" % key)
            sys.stderr.write("\nEnter help [command] for more help\n")
        else:
            sys.stdout.write("%s\n" % self.__help[input[1]])

    def installModule(self, module):
        """Call module.installModule on help and command dictionaries"""

        module.installModule(self.__commands, self.__help)

    def __quit(self, input):
        sys.exit(0)


# check to see if ~/.ciphon exists, if not try to create it and download the
# package and server lists
ciphonDir = os.path.expanduser('~/.ciphon')

if not os.path.isdir(ciphonDir):
    try:
        os.mkdir(ciphonDir)
        ftpServer = ftplib.FTP()
        try:
            ftpServer.connect('ftp.community.tummy.com')
            ftpServer.retrbinary("RETR /pub/python-packages/servers.xml",
                                 open(os.path.join(ciphonDir, 'servers.xml'), 'w').write)

            ftpServer.retrbinary("RETR /pub/python-packages/packages.xml",
                                 open(os.path.join(ciphonDir, 'packages.xml'), 'w').write)
            ftpServer.quit()
        except ftplib.socket.error:
            sys.stderr.write("Can't obtain server or package lists, giving up")
    except OSError:
        sys.stderr.write("Can't create ~/.ciphon, directory exists")
    
main  = ReadEvalLoop()

# all modules being installed need to provide a function called installModule
# this function takes two dictionaries (command, help) and inserts entries
# of the form command => function for the command dictionary and topic => helptext

# to install a module just import it and call its main.installModule(modulename)

# the installModule function installs the toplevel commands for each
# module

main.installModule(install)
main.installModule(config)
main.installModule(check)

# grab new versions of servers.xml and packages.xml
if config.getConfig().getParameter('autoupdate') == 'true':
    sys.stdout.write("Updating servers.xml and packages.xml, please wait\n")
    ftpServer = ftplib.FTP()
    try:
        ftpServer.connect('ftp.community.tummy.com')
        ftpServer.retrbinary("RETR /pub/python-packages/servers.xml",
                             open(os.path.join(ciphonDir, 'servers.xml'), 'w').write)

        ftpServer.retrbinary("RETR /pub/python-packages/packages.xml",
                             open(os.path.join(ciphonDir, 'packages.xml'), 'w').write)
        ftpServer.quit()
    except ftplib.socket.error:
        sys.stderr.write("Can't obtain server or package lists, giving up")
config.getConfig().writeConfig()
main.begin()

